#!/usr/local/bin/perl

##########################################################################
# 							Michael Grazier                                          #
#						   graz0018@d.umn.edu						                             #
#                G-Sets (Release)                                        #
#							   05/07/2004					                              			 #
#                                                                        #
# Usage : gsets.pl <term1> <term2> <N>                                   #
#                                                                        #
##########################################################################

use SOAP::Lite;
use WordNet::QueryData;

# Configuration
$key   = "X+6pp69QFHI4uxnwDxFvvFRKNY0XZIHv";

# Initialize with local SOAP::Lite file
$service = SOAP::Lite->service("file:GoogleSearch.wsdl");                       # GoogleSearch.wsdl must be in same directory as gsets.pl

# Initialize WordNet::Querydata object
$wn = WordNet::QueryData->new;

# Initialization
@results = ();
%hypernyms = ();
%hyponyms = (); $counter = 0;

$userTerm1 = shift @ARGV; $userTerm2 = shift @ARGV;	$N = shift @ARGV;							# get user's term from command line

$searchString = "\"such * as $userTerm1, $userTerm2\"";
$results[0] = doSearch($searchString);

$searchString = "\"$userTerm1, $userTerm2 or other *\"";
$results[1] = doSearch($searchString);

$searchString = "\"$userTerm1, $userTerm2 and other *\"";
$results[2] = doSearch($searchString);

#$searchString = "\"* including $userTerm1 * $userTerm2\"";
#$results[3] = doSearch($searchString);

$maxResultsCount = 0;
$maxResultsIndex = 0;

for ($i = 0; $i < 3; $i++) {
	if ($results[$i]->{estimatedTotalResultsCount} > $maxResultsCount) {          # find which search contains the most results.
		$maxResultsCount = $results[$i]->{estimatedTotalResultsCount};
		$maxResultsIndex = $i;
		$patternType = $i;														                               # keep track of which pattern we're using
	}	#end if
}	#end for
print "ESTIMATED NUMBER OF RESULTS: $maxResultsCount\n";
if ($maxResultsCount > 0) {
	for ($i = 0; $i < 10; $i++) {
	    $snippet = $results[$maxResultsIndex]->{resultElements}->[$i]->{snippet};
    	@sentences = ($snippet =~ m/(\S.*?[!?.])(?=\s)/g);                    	  # seperate the snippet into individual sentences.
		foreach $sent (@sentences) {
			if ($sent =~ m/$userTerm1/) {                                             # if the sentence contains our first term
				$snippet = $sent;                                                       # let's assume that it contains our pattern we're looking for
			}	#end if
		}	#end foreach


	$snippet =~ s/<.*?>//g;                                                       # hackishly remove HTML tags
    parseSnippet($snippet);

    }	#end for
    $hyponyms{to}=0; $hyponyms{this}=0; $hyponyms{that}=0; $hyponyms{an}=0; $hyponyms{any}=0; $hyponyms{all}=0; $hyponyms{by}=0; $hyponyms{such}=0; $hyponyms{or}=0; $hyponyms{from}=0; $hyponyms{be}=0; $hyponyms{on}=0; $hyponyms{you}=0; $hyponyms{your}=0; $hyponyms{a}=0; $hyponyms{if}=0; $hyponyms{as}=0; $hyponyms{in}=0; $hyponyms{the}=0; $hyponyms{and}=0; $hyponyms{is}=0; $hyponyms{with}=0; $hyponyms{of}=0; $hyponyms{for}=0; print "(Possibly) Related Words: \n";
    @hyponymList = sort { $hyponyms{$b} cmp $hyponyms{$a} } keys %hyponyms;     # sort the hash into an array descending order
    foreach $asdf (@hyponymList) {
    	print "\t$asdf \n"; $counter++; if ($counter == $N+1) {last;}
    }

	print "WordNet's Coordinate Terms for $userTerm1:\n\n";
    
    $file = "out";
    system("wn $userTerm1 -coorn > $file");

    open(OUT, $file) || die "Could not open file :$file\n";

    while(<OUT>) {
      chomp;
      print "$_\n";
    } #end while

}	#end if

sub parseSnippet {
	if ($patternType == 0) {														                            # "Such * as, *.."
 	    @temp = ();
 	    $_[0] =~ s/[?;:!,.'"]//g;
		$_[0] =~ m/such/;
		$hyper = $3;
		$hypo = $1;
		@temp = split/ /, $hyper;
		$word = shift @temp;
		$word = shift @temp;
		$hypernyms{$word}++;														                            # hypernym is first word after such

		$_[0] =~ m/\was/;
		@temp = ();
		@temp = split//, $hypo;
		foreach $hyponym (@temp) {
			$hyponyms{lc($hyponym)}++;
		}
	}	#end if


	if ($patternType == 1) {
		@temp = ();
        $_[0] =~ s/[?;:!,.'"]//g;
		$_[0] =~ m/(.*)(or other)(.*)/;
		$hyper = $3;
		$hypo = $1;
		@temp = split/ /, $hyper;
		$word = shift @temp;
		$word = shift @temp;
		$hypernyms{$word}++;

		@temp = ();
		@temp = split/ /, $hypo;
		foreach $hyponym (@temp) {
			$hyponyms{lc($hyponym)}++;
		}	#end foreach
	}	#end if

	if ($patternType == 2) {
		@temp = ();
		$_[0] =~ s/[?;:!,.'"]//g;												# remove punctuation
		$_[0] =~ m/(.*)(and other)(.*)/;
		$hyper = $3;
		$hypo = $1;
		@temp = split/ /, $hyper;
		$word = shift @temp;
		$word = shift @temp;
		$word=~s/^\s+//;
        $word=~s/\s+$//;
		$hypernyms{$word}++;

		@temp = ();
		@temp = split/ /, $hypo;
		foreach $hyponym (@temp) {
            $hyponyms{lc($hyponym)}++;
        }
    }
}

sub doSearch {
	$result = $service->doGoogleSearch(
							$key,                           # key
					        $_[0],        	    	        # search query
					        0, 		                        # start results
							10,                             # max results
							"false",                        # filter: boolean
							"",                             # restrict (string)
							"false",                        # safeSearch: boolean
							"",                             # lr
   							"",                           	# ie
							""              	         	# oe
							);
	return $result;
}	#end doSearch
